@@ -9,12 +9,12 @@ from ujson import dumps
 from colorama import Fore, Style
 from octopus import TornadoOctopus
 from octopus.limiter.redis.per_domain import Limiter
-from sqlalchemy.exc import OperationalError
 
 from holmes import __version__
 from holmes.reviewer import Reviewer, InvalidReviewError
 from holmes.utils import load_classes, count_url_levels
-from holmes.models import Settings, Worker, Page
+from holmes.models import Settings, Worker, Page, Domain
+from holmes.models import Limiter as LimiterModel
 from holmes.cli import BaseCLI
 
 
@@ -42,7 +42,9 @@ class BaseWorker(BaseCLI):
 
     def update_otto_limiter(self):
         domains = self.cache.get_domain_limiters()
-        self.otto.limiter.update_domain_definitions(*domains)
+
+        if hasattr(self.otto, 'limiter'):
+            self.otto.limiter.update_domain_definitions(*domains)
 
     def start_otto(self):
         self.info('Starting Octopus with %d concurrent threads.' % self.options.concurrency)
@@ -115,9 +117,6 @@ class HolmesWorker(BaseWorker):
         self.connect_to_redis()
         self.start_otto()
 
-        self.facters = self._load_facters()
-        self.validators = self._load_validators()
-
         self.fact_definitions = {}
         self.violation_definitions = {}
 
@@ -189,7 +188,6 @@ class HolmesWorker(BaseWorker):
                 page_uuid=job['page'],
                 page_url=job['url'],
                 page_score=job['score'],
-                increase_lambda_tax_method=self._increase_lambda_tax,
                 config=self.config,
                 validators=self.validators,
                 facters=self.facters,
@@ -205,50 +203,45 @@ class HolmesWorker(BaseWorker):
 
             reviewer.review()
 
-    def _increase_lambda_tax(self, tax):
-        tax = float(tax)
+    def _ping_api(self):
+        self.debug('Pinging that this worker is still alive...')
+
         for i in range(3):
             self.db.begin(subtransactions=True)
             try:
-                self.db.query(Settings).update({'lambda_score': Settings.lambda_score + tax})
+                # FIXME: this statement works only in MySQL.
+                query_params = {
+                    'uuid': self.uuid,
+                    'dt': datetime.utcnow(),
+                    'current_url': self.working_url
+                }
+
+                self.db.execute(
+                    'INSERT INTO workers (uuid, last_ping, current_url) ' \
+                    'VALUES (:uuid, :dt, :current_url) ON DUPLICATE KEY ' \
+                    'UPDATE last_ping = :dt, current_url = :current_url',
+                    query_params
+                )
+
+                self.db.flush()
                 self.db.commit()
-                break
+
+                self.publish(dumps({
+                    'type': 'worker-status',
+                    'workerId': str(self.uuid)
+                }))
+
+                return True
+
             except Exception:
                 err = sys.exc_info()[1]
                 if 'Deadlock found' in str(err):
                     self.error('Deadlock happened! Trying again (try number %d)! (Details: %s)' % (i, str(err)))
                 else:
                     self.db.rollback()
-                    raise
+                    self.error("Could not ping API due to error: %s" % str(err))
 
-    def _ping_api(self):
-        self.debug('Pinging that this worker is still alive...')
-
-        worker = Worker.by_uuid(self.uuid, self.db)
-
-        self.db.begin(subtransactions=True)
-
-        try:
-            if worker:
-                worker.last_ping = datetime.utcnow()
-                worker.current_url = self.working_url
-            else:
-                worker = Worker(uuid=self.uuid, current_url=self.working_url)
-                self.db.add(worker)
-            self.db.flush()
-            self.db.commit()
-        except OperationalError:
-            exc = sys.exc_info()[1]
-            self.db.rollback()
-            self.error("Could not ping API due to error: %s" % str(exc))
-            return False
-
-        self.publish(dumps({
-            'type': 'worker-status',
-            'workerId': str(worker.uuid)
-        }))
-
-        return True
+        return False
 
     def handle_limiter_miss(self, url):
         self._ping_api()
@@ -283,6 +276,8 @@ class HolmesWorker(BaseWorker):
 
     def _start_job(self, url):
         self.update_otto_limiter()
+        if not self._verify_workers_limits(url):
+            return False
 
         self.working_url = url
 
@@ -295,6 +290,10 @@ class HolmesWorker(BaseWorker):
 
         return True
 
+    def _verify_workers_limits(self, url, avg_links_per_page=10):
+        active_domains = Domain.get_active_domains(self.db)
+        return LimiterModel.has_limit_to_work(self.db, active_domains, url, avg_links_per_page)
+
     def _complete_job(self, lock, error=None):
         self.working_url = None
         worker = Worker.by_uuid(self.uuid, self.db)
